@@ -17,15 +17,13 @@ class Dashboard(object):
     app = ''
     template_name = 'responsive_dashboard/dashboard.html'
 
-    def __init__(self):
-        responsive_dashboards['{0}__{1}'.format(self.app, self.title)] = self
-
 
 class Dashlet(TemplateView):
     """ Base class for dashlets
     Extends TemplateView so it could actually be a view as well.
     """
     title = None
+    verbose_name = None
     template_name = 'responsive_dashboard/dashlet.html'
     has_config = False
     template_dict = {}
@@ -54,10 +52,14 @@ class Dashlet(TemplateView):
     def set_request(self, request):
         """ Set the html request from the view """
         self.request = request
+    
+    def get_verbose_name(self):
+        if self.verbose_name:
+            return self.verbose_name
+        return self.title
 
     def _check_perm(self):
         """ Check if user has permission to view """
-        print "checkem!!!!!!!!!!!!!!"
         for perm in self.require_permissions:
             if not self.request.user.has_perm(perm):
                 return False
@@ -130,10 +132,9 @@ class ListDashlet(Dashlet):
     order_by = ()
     count = 5
     show_change = True
-    show_add = False
     show_custom_link = None
     custom_link_text = None
-    first_column_is_link = False
+    first_column_is_link = True 
     allow_multiple = True
     extra_links = {}
     
@@ -170,13 +171,18 @@ class ListDashlet(Dashlet):
                 except FieldDoesNotExist:
                     headers += [field]
 
+        opts = self.model._meta
+        has_add_permission = self.request.user.has_perm('{}.add_{}'.format(opts.app_label, opts.model_name))
+        has_change_permission = self.request.user.has_perm('{}.change_{}'.format(opts.app_label, opts.model_name))
+
         context = dict(context.items() + {
-            'opts': self.model._meta,
+            'opts': opts,
             'object_list': object_list,
             'results': results,
             'headers': headers,
             'show_change': self.show_change,
-            'show_add': self.show_add,
+            'has_add_permission': has_add_permission,
+            'has_change_permission': has_change_permission,
             'show_custom_link': self.show_custom_link,
             'custom_link_text': self.custom_link_text,
             'first_column_is_link': self.first_column_is_link,
@@ -253,25 +259,58 @@ class RssFeedDashlet(Dashlet):
         return context
 
 
+try:
+    from collections import OrderedDict
+except:
+    OrderedDict = dict # pyflakes:ignore
 
-responsive_dashboards = {}
-# code from django-admin-tools THANKS!
-# https://bitbucket.org/izi/django-admin-tools/src/e2732c0083c76862c7d014b7ab25d0dbd5e467c5/admin_tools/dashboard/registry.py
-for app in settings.INSTALLED_APPS:
-    # try to import the app
-    try:
-        app_path = import_module(app).__path__
-    except AttributeError:
-        continue
+def autodiscover():
+    """
+    Auto-discover INSTALLED_APPS report.py modules and fail silently when
+    not present. Borrowed form django.contrib.admin
+    """
+    from django.utils.importlib import import_module
+    from django.utils.module_loading import module_has_submodule
 
-    # try to find a app.dashboard module
-    try:
-        imp.find_module('dashboards', app_path)
-    except ImportError:
-        continue
+    global dashboards
 
-    # looks like we found it so import it !
-    try: # TODO find a better way to make this work
-        import_module('%s.dashboards' % app)
-    except:
-        pass
+    for app in settings.INSTALLED_APPS:
+        mod = import_module(app)
+        # Attempt to import the app's admin module.
+        try:
+            before_import_registry = copy.copy(dashboards)
+            import_module('%s.dashboards' % app)
+        except:
+            dashboards = before_import_registry
+            if module_has_submodule(mod, 'dashboards'):
+                raise
+
+
+class DashboardClassManager(object):
+    """
+    Class to handle registered dashboards class.
+    """
+    _register = OrderedDict()
+
+    def __init__(self):
+        self._register = OrderedDict()
+
+    def register(self, slug, rclass):
+        if slug in self._register:
+            raise ValueError('Slug already exists: %s' % slug)
+        setattr(rclass, 'slug', slug)
+        self._register[slug] = rclass
+
+    def get_dashboard(self, slug):
+        # return class
+        print "!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!"
+        print slug
+        print self._register.values()
+        return self._register.get(slug, None)
+
+    def get_dashboards(self):
+        # return clasess
+        return self._register.values()
+
+
+dashboards = DashboardClassManager()
